/*
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 *    BESEnsembleLibrary.java
 *    Copyright (C) 1999-2012 University of Waikato, Hamilton, New Zealand
 *
 */

package weka.classifiers.meta.baggingEnsembleSelection;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.Serializable;
import java.util.TreeSet;
import weka.classifiers.AbstractClassifier;
import weka.classifiers.Classifier;
import weka.classifiers.meta.baggingEnsembleSelection.BESEnsembleLibraryModel.BESEnsembleLibraryModelComparator;
import weka.core.RevisionHandler;
import weka.core.RevisionUtils;

/**
 *
 * @author Quan Sun (quan.sun.nz@gmail.com)
 * @author University of Waikato
 * @author Cornell University
 */
public class BESEnsembleLibrary
        implements Serializable, RevisionHandler {

    private static final long serialVersionUID = -7987178904923706760L;
    
    public static final String XML_FILE_EXTENSION = ".model.xml";
    
    public static final String FLAT_FILE_EXTENSION = ".mlf";
    
    public TreeSet m_Models;
    
    private transient PropertyChangeSupport m_LibraryPropertySupport = new PropertyChangeSupport(this);

    public BESEnsembleLibrary() {

        m_Models = new TreeSet(new BESEnsembleLibraryModelComparator());
    }

    public int size() {
        if (m_Models != null) {
            return m_Models.size();
        } else {
            return 0;
        }
    }

    public void addModel(BESEnsembleLibraryModel model) {
        m_Models.add(model);
        if (m_LibraryPropertySupport != null) {
            m_LibraryPropertySupport.firePropertyChange(null, null, null);
        }
    }

    public void addModel(String modelString) {
        m_Models.add(createModel(modelString));
        m_LibraryPropertySupport.firePropertyChange(null, null, null);
    }

    public void removeModel(BESEnsembleLibraryModel model) {
        m_Models.remove(model);
        m_LibraryPropertySupport.firePropertyChange(null, null, null);
    }

    public BESEnsembleLibraryModel createModel(Classifier classifier) {
        BESEnsembleLibraryModel model = new BESEnsembleLibraryModel(classifier);

        return model;
    }

    public BESEnsembleLibraryModel createModel(String modelString) {

        String[] splitString = modelString.split("\\s+");
        String className = splitString[0];

        String argString = modelString.replaceAll(splitString[0], "");
        String[] optionStrings = argString.split("\\s+");

        BESEnsembleLibraryModel model = null;
        try {
            model = new BESEnsembleLibraryModel(AbstractClassifier.forName(className,
                    optionStrings));
        } catch (Exception e) {
            e.printStackTrace();
        }
        return model;
    }

    public TreeSet getModels() {
        return m_Models;
    }

    public void setModels(TreeSet models) {
        m_Models = models;
        m_LibraryPropertySupport.firePropertyChange(null, null, null);
    }

    public void clearModels() {
        m_Models.clear();
        m_LibraryPropertySupport.firePropertyChange(null, null, null);
    }

    public void addPropertyChangeListener(PropertyChangeListener listener) {

        if (m_LibraryPropertySupport != null) {
            m_LibraryPropertySupport.addPropertyChangeListener(listener);

        }
    }

    public String getRevision() {
        return RevisionUtils.extract("$Revision: 6041 $");
    }
}
